import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * An ant that collects food.
 * 
 * @author Michael Kolling
 * @version 1.1
 */
public class Ant extends Creature
{
    private boolean carryingFood;
    private int numStepsSinceLastPheromoneDrop;
    private int smelledPheromone;
    
    /**
     * Create an ant with a given home hill. The initial speed is zero (not moving).
     */
    public Ant(AntHill home)
    {
        setHomeHill(home);
        carryingFood = false;
        numStepsSinceLastPheromoneDrop = 0;
        smelledPheromone = 0;
    }

    /**
     * Do what an ant's gotta do.
     */
    public void act()
    {
        if(carryingFood) {
            walkTowardsHome();
            dropPheromone();
            checkForHome();
        }
        else {
            if(smelledPheromone > 0) {
                walkAwayFromHome();
                smelledPheromone = smelledPheromone - 1;
            }
            else {
                randomWalk();
                checkForPheromone();
            }
        }
        checkForFood();
    }
    
    private void checkForHome() {
        AntHill hill = (AntHill) getOneIntersectingObject(AntHill.class);
        if (hill != null) {
            setImage("ant.gif");
            carryingFood = false;
            hill.countFood();
        }
    }
    
    private void checkForFood() {
        Food food = (Food) getOneIntersectingObject(Food.class);
        if(food != null) {
            //Greenfoot.stop();
            setImage("ant-with-food.gif");
            carryingFood = true;
            food.takeSome();
        }
    }
    
    private void dropPheromone() {
        numStepsSinceLastPheromoneDrop = numStepsSinceLastPheromoneDrop + 1;
        if( numStepsSinceLastPheromoneDrop == 18) {
            getWorld().addObject(new Pheromone(), getX(), getY());
            numStepsSinceLastPheromoneDrop = 0;
        }
    }
    
    private void checkForPheromone() {
        if(getOneIntersectingObject(Pheromone.class) != null) {
            smelledPheromone = 30;
        }
    }
    
    /**
     * Are we home?
     */
    private boolean atHome()
    {
        if (getHomeHill() != null) {
            return (Math.abs(getX() - getHomeHill().getX()) < 4) && (Math.abs(getY() - getHomeHill().getY()) < 4);
        }
        else {
            return false;
        }
        
    }
}